// =============================================================================
// RELATED SOFTWARE: Scilab ≥ 6.0.0
// PACKAGE  : read_datablock : read data embeded in bloc-comments appended in your script
// VERSION  : 1.0 - 2023-05-28
// AUTHOR   : Samuel GOUGEON - Le Mans Université
// COPYRIGHT: Samuel GOUGEON -  2023
// LICENSE : CeCILL-C : https://cecill.info/licences/Licence_CeCILL-C_V1-en.html
// =============================================================================
// <-- CLI SHELL MODE -->
// <-- NO CHECK REF -->
// <-- ENGLISH IMPOSED -->

// =================================
// Unitary tests of read_datablock()
// =================================
// ----------------------------------------------------------------------------
// Extraction from the executed script
// -----------------------------------
/* First block comment:

  3  8  2 -2  0 -7  2  9   // first row
       
  8 -2  8 -8 -5 -2  7  6   // second row
  0 -2  7 -6 -5 -8  7 -9
*/
// Let open any file after the current script:
fid = mopen("TMPDIR/cleaner.txt","a");

// Read the first block-comment of the current file:
    // as data, with default evaluation with evstr():
    // Empty or blank lines and appended comments are ignored.
ref = [3  8  2 -2  0 -7  2  9
       8 -2  8 -8 -5 -2  7  6
       0 -2  7 -6 -5 -8  7 -9 ];
r = read_datablock(1);
assert_checkequal(r, ref);
    // as literal
r = read_datablock(1, "-string");
ref = [ ""
        "  3  8  2 -2  0 -7  2  9   // first row"
        "       "
        "  8 -2  8 -8 -5 -2  7  6   // second row"
        "  0 -2  7 -6 -5 -8  7 -9"];
assert_checkequal(r, ref);

/* Second data block:
 -4 -5 -6  1  9  9  6  2
  6 -1  7  1  2  7 -6 -2
*/

// Read the second block-comment of the current file
ref = [ -4 -5 -6  1  9  9  6  2
         6 -1  7  1  2  7 -6 -2 ];
r = read_datablock(2);
assert_checkequal(r, ref);
    // as literal:
r = read_datablock(2, "-string");
ref = [ " -4 -5 -6  1  9  9  6  2"
        "  6 -1  7  1  2  7 -6 -2" ];
assert_checkequal(r, ref);

// Read a block-comment according to a keyword/tag in its headline
r = read_datablock("boolean");
ref = [ %f %f %t %f %f %t %f %f %f %f
        %f %t %f %f %t %t %t %t %f %t ];
assert_checkequal(r, ref);

r = read_datablock("boolean", "-string");
ref = ["  F F T F F T F F F F"
       "  F T F F T T T T F T" ];
assert_checkequal(r, ref);

r = read_datablock("Customers","-string");
ref = [
    "John  SMITH     49  Canada"
    "Elen  RACLIF    35  USA"
    "Richard MICHEL  17  France"
    "Katarina ORIANA 62  Brazil" ];
assert_checkequal(r, ref);
//r = msscanf(-1, ref, "%s %s %d %s");
//assert_checkequal(r(:,1),["John" "Elen" "Richard" "Katarina"]');
//assert_checkequal(r(:,2),["SMITH" "RACLIF" "MICHEL" "ORIANA"]');
//assert_checkequal(r(:,3),[49 35 17 62]');
//assert_checkequal(r(:,4),["Canada" "USA" "France" "Brazil"]');

/* Customers
John  SMITH     49  Canada
Elen  RACLIF    35  USA
Richard MICHEL  17  France
Katarina ORIANA 62  Brazil
*/
/* Some booleans
  F F T F F T F F F F
  F T F F T T T T F T
*/

// Continuation marks
// ------------------
r = read_datablock("continuation");
assert_checkequal(r, [3 8 2 -2 0 8 -2  8 -8 5  0 -1  7]);

r = read_datablock("continuation", "-string");
ref = ["  3  8  2 -2  0  .."
       "  8 -2  8 -8 ..."
       "  5  0 -1  7"];
assert_checkequal(r, ref);

r = read_datablock("continuation", "-string..");
assert_checkequal(r, "  3  8  2 -2  0    8 -2  8 -8   5  0 -1  7");

/* Third block with continuation marks (at least two dots):
  3  8  2 -2  0  ..
  8 -2  8 -8 ...
  5  0 -1  7
*/

// Inexisting blocks
// -----------------
r = read_datablock(10);
assert_checkequal(r, []);

r = read_datablock("blabla");
assert_checkequal(r, []);

mclose(fid)
// ----------------------------------------------------------------------------
// --------------------------------
// Extraction from an external file
// --------------------------------
[?, p] = libraryinfo("read_datablocklib");
File = fullfile(p, "read_datablock.sci");

r = read_datablock(File, 1, "-string")(1:2);
ref = ["    read_datablock(n)"
       "    read_datablock(tag)" ];
assert_checkequal(r, ref);

r = read_datablock(File, "booleans", "-string")
ref = ["  F F T F F T T"
       "  T F F T T F T" ];
assert_checkequal(r, ref);

r = read_datablock(File, "booleans");
assert_checkequal(r, [%f %f %t %f %f %t %t ; %t %f %f %t %t %f %t]);

r = read_datablock(File, 3, "-string");
ref = [  "  -9   9   5  -2  -7 ..."
  "  -3   4   5  -2  -4"
  ""
  "   7   4   5   3   4 .." 
  "  -6  -2  -1  -7   6" ];
assert_checkequal(r, ref);

r = read_datablock(File, 3, "-string..");
ref = ["  -9   9   5  -2  -7   -3   4   5  -2  -4"
       ""
       "   7   4   5   3   4   -6  -2  -1  -7   6" ];
assert_checkequal(r, ref);

r = read_datablock(File, 3);
ref = [-9.   9.   5.  -2.  -7.  -3.   4.   5.  -2.  -4.
        7.   4.   5.   3.   4.  -6.  -2.  -1.  -7.   6. ];
assert_checkequal(r, ref);

// Inexisting blocks
// -----------------
r = read_datablock(File, 10);
assert_checkequal(r, []);

r = read_datablock(File, "blabla");
assert_checkequal(r, []);

// ---------------------------------------------------------------------------

// ERRORS
// ------
msgRef = "read_datablock: Wrong number of input arguments: 1 to 3 expected.";
assert_checkerror("read_datablock(%t, %t, %t, %t)", msgRef);

msgRef = "read_datablock: Argument #1: integer > 0 expected.";
assert_checkerror("read_datablock(1.5)", msgRef);
assert_checkerror("read_datablock(-1)", msgRef);
assert_checkerror("read_datablock(%i)", msgRef);
msgRef = "read_datablock: Argument #1: Scalar (1 element) expected.";
assert_checkerror("read_datablock([1 2])", msgRef);
